// InkedButton.cpp : implementation file
//

// NOTE: I'm using my own bitmapped button implementation
// rather than CBitmapButton because I want to be sure the
// image is drawn with the appropriate background color.
// With CBitmapButton you must provide a bitmap texture
// that covers the entire button's surface, which I don't
// want to do in this case...  Change the Windows button
// colors and see what happens to the button...

#include "stdafx.h"
#include "DOMDemo.h"
#include "InkedButton.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CInkedButton

CInkedButton::CInkedButton()
 : m_bExpandMode(TRUE)
{
}

CInkedButton::~CInkedButton()
{
}


BEGIN_MESSAGE_MAP(CInkedButton, CButton)
	//{{AFX_MSG_MAP(CInkedButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInkedButton message handlers

void CInkedButton::DrawFocusRect(CDC& dc, CRect& rc)
{
	// Draw lines around the edge of the rectangle. Unfortunately,
	// CDC::DrawFocusRect() and ::DrawFocusRect() don't draw the
	// focus rectangle in BLACK!  We have to do it ourselves...
	dc.MoveTo(rc.left,rc.top);
	int i = rc.left;
	BOOL bDraw = FALSE; // upper-left corner pixel starts OFF
	while ( i < rc.right ) { // top
		if ( bDraw ) {
			dc.SetPixel(i,rc.top,RGB(0,0,0));
		} // if
		bDraw = ~bDraw;
		++i;
	} // while
	dc.MoveTo(rc.right-1,rc.top);
	i = rc.top; // draw the way Windows would draw using Rectangle()
	while ( i < rc.bottom ) { // right
		if ( bDraw ) {
			dc.SetPixel(rc.right-1,i,RGB(0,0,0));
		} // if
		bDraw = ~bDraw;
		++i;
	} // while
	dc.MoveTo(rc.right-1,rc.bottom-1);
	i = rc.left; // draw the way Windows would draw using Rectangle()
	while ( i < rc.right ) { // bottom
		if ( bDraw ) {
			dc.SetPixel(i,rc.bottom-1,RGB(0,0,0));
		} // if
		bDraw = ~bDraw;
		++i;
	} // while
	dc.MoveTo(rc.left,rc.top);
	i = rc.top;
	while ( i < rc.bottom ) { // left
		if ( bDraw ) {
			dc.SetPixel(rc.left,i,RGB(0,0,0));
		} // if
		bDraw = ~bDraw;
		++i;
	} // while
}

void CInkedButton::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	// Pull struct data
	CDC dcBtn;
	dcBtn.Attach(lpDrawItemStruct->hDC);
	CRect rcControl(lpDrawItemStruct->rcItem);

	// Begin by filling drawing area with button color (to erase
	// any left-over junk)
	dcBtn.FillSolidRect(rcControl,::GetSysColor(COLOR_BTNFACE));

	// Next draw appropriate 3D border
	if ( lpDrawItemStruct->itemState & ODS_SELECTED ) {
		// Pushed, so sunken 3D
		if( lpDrawItemStruct->itemState & ODS_FOCUS )
		{
			// Draw sunken focus border
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DDKSHADOW),GetSysColor(COLOR_3DDKSHADOW));
			rcControl.DeflateRect(1,1);
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DSHADOW),GetSysColor(COLOR_3DSHADOW));
			rcControl.DeflateRect(3,3);

			// Draw the focus rectangle
			DrawFocusRect(dcBtn,rcControl);
		} // if
		else {
			// Draw sunken border normally
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DSHADOW),GetSysColor(COLOR_3DHILIGHT));
			rcControl.DeflateRect(1,1);
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DDKSHADOW),GetSysColor(COLOR_3DLIGHT));
			rcControl.DeflateRect(3,3);
		} // else

		// Add offset for bitmap placement
		rcControl.OffsetRect(1,1);
	} // if
	else {
		// Released, so raised 3D
		if( lpDrawItemStruct->itemState & ODS_FOCUS )
		{
			// Draw raised focus border
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DDKSHADOW),GetSysColor(COLOR_3DDKSHADOW));
			rcControl.DeflateRect(1,1);
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DHILIGHT),GetSysColor(COLOR_3DDKSHADOW));
			rcControl.DeflateRect(1,1);
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DLIGHT),GetSysColor(COLOR_3DSHADOW));
			rcControl.DeflateRect(2,2);

			// Draw the focus rectangle
			DrawFocusRect(dcBtn,rcControl);
		} // if
		else {
			// Draw raised border normally
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DHILIGHT),GetSysColor(COLOR_3DDKSHADOW));
			rcControl.DeflateRect(1,1);
			dcBtn.Draw3dRect(rcControl,GetSysColor(COLOR_3DLIGHT),GetSysColor(COLOR_3DSHADOW));
			rcControl.DeflateRect(3,3);
		} // else
	} // else

	// Draw appropriate image (should draw transparently). Note
	// the rectangle work is to properly center the image.
	CPoint pt(rcControl.left + ((rcControl.Width() - 13) / 2), // 13 is width of bitmap
			  rcControl.top + ((rcControl.Height() - 14) / 2)); // 14 is height of bitmap
	m_CILButton.Draw(&dcBtn,
					 m_bExpandMode ? 0 : 1,
					 pt,
					 ILD_NORMAL);
	
	// Unhook the DC...
	dcBtn.Detach();
}

void CInkedButton::PreSubclassWindow() 
{
	// Initialize image list
	CBitmap bmpIcons, bmpMask;
	bmpIcons.LoadBitmap(IDB_BTNICONS);
	bmpMask.LoadBitmap(IDB_BIMASK);
	m_CILButton.Create(13,14,ILC_COLOR8|ILC_MASK,2,0);
	m_CILButton.Add(&bmpIcons,&bmpMask);
	m_CILButton.SetBkColor(CLR_NONE);

	CButton::PreSubclassWindow();
}
